# 🔐 Cryptrack

`Cryptrack` — лёгкая TypeScript-библиотека для безопасного обмена данными между сервисами с использованием ECDH (`secp256k1`) и симметричного шифрования (AES).

## ✨ Возможности

- ECDH-вычисление общего секрета между двумя ключами
- Шифрование и расшифровка с помощью AES
- Хранение sharedKey в in-memory Map
- Минимум зависимостей

---

## 📦 Установка

```bash
npm install @crysteam/cryptrack
````

---

## 🚀 Быстрый старт

```ts
import { Cryptrack } from '@crysteam/cryptrack';
import { ec } from 'elliptic';

// Создаём экземпляр Cryptrack
const cryptrack = new Cryptrack();

// Имитируем стороннего клиента
const client = new ec('secp256k1').genKeyPair();
const clientPublicKey = client.getPublic('hex');

// Регистрируем клиента
cryptrack.register('client-1', clientPublicKey);

// Шифруем сообщение
const encrypted = cryptrack.encrypt('client-1', 'Привет, клиент!');
console.log('🔐 Encrypted:', encrypted);

// Расшифровываем
const decrypted = cryptrack.decrypt('client-1', encrypted);
console.log('📬 Decrypted:', decrypted);

// 🔑 Работа с токенами
const token = await cryptrack.signToken('client-1', { role: 'admin' }, '30m');
const payload = await cryptrack.verifyToken(token);
```

---

## 🧩 Методы

### `register(id: string, publicKeyHex: string): string`

Регистрирует клиента и сохраняет sharedKey на основе его публичного ключа.

---

### `getKey(id: string): string | undefined`

Возвращает ранее сохранённый sharedKey по идентификатору.

---

### `encrypt(id: string, data: string): string`

Шифрует строку `data` с помощью AES и sharedKey, связанного с `id`.

---

### `decrypt(id: string, encrypted: string): string`

Расшифровывает строку с использованием sharedKey, связанного с `id`.

---

### `getPublicKey(): string`

Возвращает публичный ключ текущего Cryptrack-инстанса (hex).

---

## 📄 Лицензия
MIT
