import { ec as EC } from 'elliptic';
import { AES, enc, SHA256 } from 'crypto-js';
import { generateKeyPair, JWTPayload, jwtVerify, SignJWT } from 'jose';

export class Cryptrack {
    private readonly _curve = new EC('secp256k1');
    private readonly _keys = this._curve.genKeyPair();
    private readonly _store = new Map<string, string>();
    private readonly _jwtKeyPromise = generateKeyPair('HS256').then(kp => kp.privateKey);

    register(id: string, publicKeyHex: string): string {
        try {
            const peerPubKey = this._curve.keyFromPublic(publicKeyHex, 'hex').getPublic();
            const sharedSecret = this._keys.derive(peerPubKey).toString(16);
            const sharedKey = SHA256(sharedSecret).toString();

            this._store.set(id, sharedKey);
            return sharedKey;
        } catch (err) {
            throw new Error(`Invalid public key: ${(err as Error).message}`);
        }
    }

    getKey(id: string): string | undefined {
        return this._store.get(id);
    }

    encrypt(id: string, data: string): string {
        const key = this.getKey(id);
        if (!key) throw new Error(`Shared key for ID "${id}" not found`);
        return AES.encrypt(data, key).toString();
    }

    decrypt(id: string, encrypted: string): string {
        const key = this.getKey(id);
        if (!key) throw new Error(`Shared key for ID "${id}" not found`);
        try {
            const decrypted = AES.decrypt(encrypted, key).toString(enc.Utf8);
            if (!decrypted) throw new Error('Failed to decode decrypted data');
            return decrypted;
        } catch (err) {
            throw new Error(`Decryption failed: ${(err as Error).message}`);
        }
    }

    getPublicKey(): string {
        return this._keys.getPublic('hex');
    }

    async signToken(id: string, payload: Record<string, any>, expires: string = '1h'): Promise<string> {
        const key = await this._jwtKeyPromise;

        return new SignJWT(payload)
            .setProtectedHeader({ alg: 'HS256' })
            .setSubject(id)
            .setIssuedAt()
            .setExpirationTime(expires)
            .sign(key);
    }

    async verifyToken(token: string): Promise<{ id: string; payload: JWTPayload }> {
        const key = await this._jwtKeyPromise;

        const { payload } = await jwtVerify(token, key);
        if (!payload.sub) throw new Error('Token missing subject');

        return {
            id: payload.sub,
            payload,
        };
    }
}
